<a name="logo"/>
<div align="center">
<img src="images/Logo_PiGx.png" alt="PiGx Logo"  width="30%" height="30%" ></img>
</a>
</div>

**Copyright 2017-2018: Bora Uyar, Jona Ronen, Ricardo Wurmus.**
**This work is distributed under the terms of the GNU General Public License, version 3 or later.  It is free to use for all purposes.**

-----------

# Documentation

Please see our documentation in order to find out more about how to install and run the PiGx-RNAseq pipeline:
http://bioinformatics.mdc-berlin.de/pigx_docs/pigx-rna-seq.html

# Sample Reports

See sample HTML reports produced by PiGx pipelines:
http://bioinformatics.mdc-berlin.de/pigx/supplementary-materials.html

# Citing PiGx-RNAseq

Wurmus, Ricardo, Bora Uyar, Brendan Osberg, Vedran Franke, Alexander Gosdschan, Katarzyna Wreczycka, Jonathan Ronen, and Altuna Akalin. 2018. “PiGx: Reproducible Genomics Analysis Pipelines with GNU Guix.” GigaScience, October. https://doi.org/10.1093/gigascience/giy123.

# Getting help

If you have any questions please e-mail: pigx@googlegroups.com or use the web form to ask questions https://groups.google.com/forum/#!forum/pigx/. 

If you run into any bugs, please open an issue here: https://github.com/BIMSBbioinfo/pigx_rnaseq/issues. 

# Links

- [The Bioinformatics and Omics Data Science Platform](https://bioinformatics.mdc-berlin.de) 
- [The German Network for Bioinformatics Infrastructure](https://www.denbi.de)

-----------
